/*
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        28.05.2004
 Description:    Dieses Programm dient als Beispiel zur Ansteuerung einer MMC/SD-Memory-Card.
				 Zum Zugriff auf eine MMC/SD-Karte, mu man nur die Datei mmc.c
				 in sein eigenes Programm einfgen.
*/

#include <stdio.h>
#include <avr/io.h>


//Anpassen der Register wenn ein ATMega128 benutzt wird
#if defined (__AVR_ATmega128__)
#	define USR UCSR0A
#	define UCR UCSR0B
#	define UDR UDR0
#	define UBRR UBRR0L
#	define EICR EICRB
#endif

//Anpassen der Register wenn ein ATMega32 benutzt wird
#if defined (__AVR_ATmega32__)
#	define USR UCSRA
#	define UCR UCSRB
#	define UBRR UBRRL
#	define EICR EICRB
#endif

//Der Quarz auf dem Experimentierboard hat eine Frequenz von 14.318MHz
#define sysclk	14238281	//Quarz Frequenz in Hz

//Die Baud_Rate der Seriellen Schnittstelle ist 9600 Baud
#define baud_rate 9600		//Baud Rate fr die Serielle Schnittstelle	

//#include <mmc_lib.c> //other mmc library
#include <mmc.c> //meine mmc library
#include <fat.c>

//Installation der Seriellen Schnittstelle
void IOInit (void)
{
	//Enable TXEN im Register UCR TX-Data Enable
	outp ((1 << TXEN),UCR);
	//Teiler wird gesetzt 
	outp ((sysclk / (baud_rate * 16L) - 1) , UBRR);
}


//Routine fr printf
int uart_putchar (char c)
{
	if (c == '\n')
		uart_putchar('\r');
	//Warten solange bis Zeichen gesendet wurde
	loop_until_bit_is_set(USR, UDRE);
	UDR = c;
	return (0);
}

//Hauptprogramm
int main (void)
{
	//Initzialisierung der seriellen Schnittstelle
	IOInit();
	
	//ffnet einen kanal fr printf
	fdevopen (uart_putchar, NULL, 0);


	//Initialisierung der MMC/SD-Karte
	printf ("System OK\n\n");
	char Timeout = 0;
	char a = 1;	
	while (a !=0) /*ist der Rckgabewert ungleich NULL ist ein Fehler aufgetreten
					MMC/SD-Karte wird dann neu Initialisiert*/
	{
		a = mmc_init();	
		Timeout++;
		/*if (Timeout == 200)
			{
			printf ("Keine MMC/SD-Karte gefunden!! %x\n",a);
			break;
			}
		*/
	}

	//Initialisierung der MMC/SD-Karte ENDE!
	printf("Karte gefunden!!");	

	//Ausgabe des Root Directory
	unsigned char Buffer[514];
	int Cluster;
	printf("\n\n\n");
	for (int a = 1;a < 512;a++)
	{
		Cluster = Read_Root_Dir_Ent(a,Buffer);
			if (Cluster == 0xffff)
			{
				break;
			}
		printf("Cluster = %4x FileName =",Cluster);
		printf(Buffer);
		printf("\n");
	}


	//Lade Fat Tabelle fr das 3 File in den Speicher 
	char BufferDaten[512];
	Cluster = Read_Root_Dir_Ent(3,Buffer); //Holt sich Daten fr 3 File aus Root Directory
	char ClusterSize = ClusSiz(BufferDaten);
	int ClusterOffset = ClusOff(BufferDaten);
	
	Load_File (Cluster,Buffer); //Auslesen der FAT Eintrge fr das File

	//Lese File und gibt es auf der seriellen Schnittstelle aus
	for (int b = 0;b<52;b++)
	{
		Read_File (Buffer,BufferDaten,b,ClusterSize,ClusterOffset);
		for (int a = 0;a<512;a++)
		{
			printf ("%c",BufferDaten[a]);
		}
	}
	
/*
	printf ("\nAusgabe der FAT Tabelle fr 1 File\n");
	for (int a = 0;a<512;a = a+4)
	{
	printf ("%x %x | %x %x \n",Buffer[a],Buffer[a+1],Buffer[a+2],Buffer[a+3]);
	}
*/

	printf("FERTIG!!\n");
	//Hauptprogramm luft stndig in einer schleife und macht nichts
	while (1)
		{
		}
return (1);
}

