/*-------------------------------------------------
	 mmc_lib.h C-header file for AVRGCC
	------------------------------------
	Interface for MultiMediaCard via SPI
	
	Thanks to:
		Ulrich Radig for his code!
		Simon Lehmayr for good ideas
	
	Author:
		Stefan Seegel
		dahamm@gmx.net
		ICQ UIN 14964408
		
	Version: 1.1 / 29.07.2004
	
	History:
		1.1 / 29.07.2004:
			-Removed typedef struct for CSD
			-Added different init-clock rate
			
		1.0 / 28.07.2004:
			-First release
	
	
	
	
	Stefan Seegel 7/2004
---------------------------------------------------*/

//Port & Pin definitions. Be sure to use a pin of the same port like SPI for CS (Chip Select) !
//Settings below are recommended for a MEGA16
#define MMC_PORT PORTB
#define MMC_DDR DDRB

#if defined (__AVR_ATmega128__)
	#define SPI_MISO	3		//DataOut of MMC 
	#define SPI_MOSI	2		//DataIn of  MMC
	#define SPI_CLK  	1		//Clock of MMC
	#define MMC_CS		4		//ChipSelect of MMC
	#define SPI_SS		0		//Nicht Benutz mu aber definiert werden
#endif

#if defined (__AVR_ATmega32__)
	#define SPI_MISO	6		//DataOut of MMC 
	#define SPI_MOSI	5		//DataIn of  MMC
	#define SPI_CLK  	7		//Clock of MMC
	#define MMC_CS		3		//ChipSelect of MMC
	#define SPI_SS		4		//Nicht Benutz mu aber definiert werden
#endif

#define MMC_Direction_REG	DDRB

/*Clockrate while initialisation / after initialisation
	SPI2X	SPR1	SPR0	SCK Frequency
	0		0		0		fosc/4
	0		0		1		fosc/16
	0		1		0		fosc/64
	0		1		1		fosc/128
	1		0		0		fosc/2
	1		0		1		fosc/8
	1		1		0		fosc/32
	1		1		1		fosc/64
*/

#define SPI_INIT_CLOCK 1<<SPR1 | 0<<SPR0
#define SPI_CLOCK 0<<SPR1 | 1<<SPR0

#define SPI_DOUBLE_SPEED 0 //0: normal speed, 1: double speed


extern unsigned char mmc_init(void);
/*			
*		Call mmc_init one time after a new card has been connected to the C's SPI bus!
*	
*		return values:
*			0: MMC initialized successfully
*			1: Error while trying to reset MMC
*			2: Error/Timeout while trying to initialize MMC
*/


extern unsigned char mmc_read_sector(unsigned long sector, char *buffer);
/*	
*		mmc_read_sector reads a block of 512 bytes from mmc.
*		Make sure that buffer points to an allocated memory with at least 512 bytes!
*
*		Parameters:
*			sector: specifies which sector to be read
*			buffer:	pointer to allocated memory where to store data
*
*		Return values:
*			0: Sector read successfully
*			1: No response after read command from MMC
*			2: No startbyte received
*/

extern unsigned char mmc_write_sector(unsigned long sector, char *buffer);
/*
*		mmc_write_sector writes a block of 512 bytes to mmc.
*
*		Parameters:
*			sector: specifies which sector to be read
*			buffer:	pointer to data to write in MMC
*
*		Return Values:
*			0: Sector written successfully
*			1: No response after write command from MMC
*			2: MMC busy timeout after writing block
*/

extern unsigned char mmc_read_csd(char *buffer);
/*	
*		reads the CSD (Card Specific Data) register from MMC.
*		Make sure that buffer points to an allocated memory with at least 16 bytes!

*
*		Parameters:
*			buffer:	pointer to allocated memory where to store data
*
*		Return values:
*			0: block read successfully
*			1: No response after read command from MMC
*			2: No startbyte received
*/

extern unsigned char mmc_read_cid(char *buffer);
/*	
*		reads the CID (Card Identification) register from MMC.
*		Make sure that buffer points to an allocated memory with at least 16 bytes!

*
*		Parameters:
*			buffer:	pointer to allocated memory where to store data
*
*		Return values:
*			0: block read successfully
*			1: No response after read command from MMC
*			2: No startbyte received
*/
